@@ -1,8 +1,11 @@
+from __future__ import annotations
+
 from typing import Final
 from mypy_extensions import u8
 
 from librt.internal import (
-    Buffer,
+    ReadBuffer,
+    WriteBuffer,
     write_bool, read_bool,
     write_str, read_str,
     write_float, read_float,
@@ -18,21 +21,20 @@ TAG_B: Final[Tag] = 255
 TAG_SPECIAL: Final[Tag] = 239
 
 
-def test_buffer_basic() -> None:
+def test_cache_version() -> None:
     assert cache_version() == 0
-    b = Buffer(b"foo")
-    assert b.getvalue() == b"foo"
 
 
-def test_buffer_empty() -> None:
-    b = Buffer(b"")
+def test_buffer_write_and_read_int() -> None:
+    b = WriteBuffer()
     write_int(b, 42)
-    b = Buffer(b.getvalue())
-    assert read_int(b) == 42
+    rb = ReadBuffer(b.getvalue())
+    assert read_int(rb) == 42
 
 
 def test_buffer_roundtrip() -> None:
-    b = Buffer()
+    b: WriteBuffer | ReadBuffer
+    b = WriteBuffer()
     write_str(b, "foo")
     write_bool(b, True)
     write_str(b, "bar" * 1000)
@@ -59,7 +61,7 @@ def test_buffer_roundtrip() -> None:
     write_int(b, 536860912)
     write_int(b, 1234567891)
 
-    b = Buffer(b.getvalue())
+    b = ReadBuffer(b.getvalue())
     assert read_str(b) == "foo"
     assert read_bool(b) is True
     assert read_str(b) == "bar" * 1000
@@ -88,48 +90,51 @@ def test_buffer_roundtrip() -> None:
 
 
 def test_buffer_int_size() -> None:
+    b: WriteBuffer | ReadBuffer
     for i in (-10, -9, 0, 116, 117):
-        b = Buffer()
+        b = WriteBuffer()
         write_int(b, i)
         assert len(b.getvalue()) == 1
-        b = Buffer(b.getvalue())
+        b = ReadBuffer(b.getvalue())
         assert read_int(b) == i
     for i in (-100, -11, 118, 12344, 16283):
-        b = Buffer()
+        b = WriteBuffer()
         write_int(b, i)
         assert len(b.getvalue()) == 2
-        b = Buffer(b.getvalue())
+        b = ReadBuffer(b.getvalue())
         assert read_int(b) == i
     for i in (-10000, 16284, 123456789):
-        b = Buffer()
+        b = WriteBuffer()
         write_int(b, i)
         assert len(b.getvalue()) == 4
-        b = Buffer(b.getvalue())
+        b = ReadBuffer(b.getvalue())
         assert read_int(b) == i
 
 
 def test_buffer_int_powers() -> None:
     # 0, 1, 2 are tested above
-    for p in range(2, 9):
-        b = Buffer()
+    b: WriteBuffer | ReadBuffer
+    for p in range(2, 100):
+        b = WriteBuffer()
         write_int(b, 1 << p)
         write_int(b, -1 << p)
-        b = Buffer(b.getvalue())
+        b = ReadBuffer(b.getvalue())
         assert read_int(b) == 1 << p
         assert read_int(b) == -1 << p
 
 
 def test_buffer_str_size() -> None:
+    b: WriteBuffer | ReadBuffer
     for s in ("", "a", "a" * 117):
-        b = Buffer()
+        b = WriteBuffer()
         write_str(b, s)
         assert len(b.getvalue()) == len(s) + 1
-        b = Buffer(b.getvalue())
+        b = ReadBuffer(b.getvalue())
         assert read_str(b) == s
 
     for s in ("a" * 118, "a" * 16283):
-        b = Buffer()
+        b = WriteBuffer()
         write_str(b, s)
         assert len(b.getvalue()) == len(s) + 2
-        b = Buffer(b.getvalue())
+        b = ReadBuffer(b.getvalue())
         assert read_str(b) == s
